#include <signal.h>
#include "stock_s.hh"
#include "feed_c.hh"

class Consumer: public StockConsumer_impl, public CORBA::ClientEventHandler
{
    private:
	Feed 		*_feedHandler;
	CORBA::String	_interestedStock;
    protected:
	void bind_succeeded(const CORBA::Object *) {
		cout << "Consumer:: Bound to Feed Handler." << endl; 
		if ( _feedHandler )
			_feedHandler->register_interest(this, _interestedStock);
	}

	void rebind_succeeded(const CORBA::Object *) {
		cout << "Consumer:: Rebound to a new Feed Handler." << endl; 
		if ( _feedHandler )
			_feedHandler->register_interest(this, _interestedStock);
	}

	void bind_failed(const CORBA::Object *, const CORBA::Exception *exp) {
		cout << "Bind to feed handler failed. " << endl;
		cout << "Exception: " << exp->_id() << endl;
	}
	void server_aborted(const CORBA::Object *) {
		cout << "Feed Handler is down." << endl;
	}

	void  receive(const StockQuote& quote) {
		cout << "Received the following quote from FeedHandler" << endl;
		cout << quote << endl;
	}
	
    public:
	Consumer(const char *feed_handler_name, const char *stock_name);
	~Consumer() { 
		_feedHandler->_release(); 
	}

};


Consumer::Consumer(const char *feed_handler_name, const char  *stock_name)
		: _interestedStock(stock_name)
{
	CORBA::BindOptions options(1, 1, CORBA::ORB::_ORB_POSTMODERN, this);
	_feedHandler = NULL;
	_feedHandler = Feed::_bind(feed_handler_name, NULL, &options);
	if ( _feedHandler == NULL) {
		cout << "Consumer:  Error doing a deferred binding to Feed"
				<< endl;
		exit(1);
	}
	else if ( _feedHandler->_is_bound() )
		_feedHandler->register_interest(this, _interestedStock);
}

void sighandler(int)
{
	exit(0);
}

int main(int argc, char **argv)
{
	signal(SIGINT, sighandler);
	if ( argc < 3 ) {
		cout << "Usage: consumer 'feed handler name' 'stock name'"
			<< endl;
		return 0;
	}
	CORBA::BOA::scope(CORBA::BOA::SCOPE_LOCAL); // Transient Objects
	Consumer consumer(argv[1], argv[2]);
	
	CORBA::BOA::impl_is_ready();
	return 0;
}


