#ifndef _any_h
#define _any_h
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/****************************************************************************/

class Any: public DSResource
{
	DECLARE_NCOBJECT_CLASS(Any);
    protected:
	Value		*_value;
    public:
	Any(); 
	Any(const Any& a);
        Any(Value *value);
	Any(const Value& value);
        Any(const TypeCode& code);
	Any(TypeCode::TCKind kind); 
	~Any(); 
	void  copyFrom(NCistream& strm);
	void  veryPrettyPrint(NCostream& strm, int spaces) const;

        const TypeCode* type() const;
	void  type(const TypeCode& code);
	TypeCode::TCKind kind() const;

	Value  *value() { return _value; }
	const Value *value() const { return _value; }
	void value(const Value& );
	void value(Value *);

	Boolean is_primitive() const;
	Long	member_count() const;


        short           shortValue() const;
        long            longValue() const;
        unsigned short  ushortValue()const;
        unsigned long   ulongValue()const;
        float           floatValue( ) const;
        double          doubleValue() const;
#if !defined(CRAY)
        unsigned char   ucharValue() const;
#endif
        char            charValue() const;
        const char *    stringValue() const;
        const TypeCode&	typeCodeValue() const;
        Object		*objrefValue() const;

        void            shortValue(short);
        void            longValue(long);
        void            ushortValue(unsigned short);
        void            ulongValue(unsigned long);
        void            floatValue(float);
        void            doubleValue(double);
#if !defined(CRAY)
        void            ucharValue(unsigned char);
#endif
        void            charValue(char);
        void            stringValue(const char*);
        void            typeCodeValue(const TypeCode& );
        void            objrefValue(Object *obj);

	operator 	short() const { return shortValue(); }
	operator 	long() const { return longValue(); }
	operator 	unsigned short() const { return ushortValue(); }
	operator 	unsigned long () const { return ulongValue(); }
	operator 	float() const { return floatValue(); }
	operator 	double() const { return doubleValue(); }
	operator 	char() const { return charValue(); }
#if !defined(CRAY)
	operator 	unsigned char() const { return ucharValue(); }
#endif
	operator 	const char *() const { return stringValue(); }
	const Value& 	operator[](Long i) const; 
	Value& 		operator[](Long i);
	const Value& 	member(const char *member_name) const;
	Value& 		member(const char *member_name);
	operator	Object *() const { return objrefValue(); }


	void		operator=(const Any&);
	void 		operator=(short val) { shortValue(val); }
	void 		operator=(long val) { longValue(val); }
	void 		operator=(unsigned short val) {ushortValue(val); }
	void 		operator=(unsigned long val) {ulongValue(val); }
	void 		operator=(float val) { floatValue(val); }
	void 		operator=(double val) { doubleValue(val);}
	void 		operator=(char val) { charValue(val);}
#if !defined(CRAY)
	void 		operator=(unsigned char val) {ucharValue(val);}
#endif
	void 		operator=(const char * val) {stringValue(val);}
	void 		operator=(const TypeCode& val ) {typeCodeValue(val);}
	void 		operator=(Object *val ) {objrefValue(val);}

	void 		get(NCistream& strm);
	void		put(NCostream& strm) const;
};

#endif
