#ifndef _boa_h
#define _boa_h
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/****************************************************************************/

typedef void (*_PMCSkelFunc)(void *, CORBA::MarshalStream& strm,
			CORBA::Environment& env, CORBA::Principal *principal);


class BOA
{
    public:
	enum RegistrationScope {			// Registration scope
		SCOPE_GLOBAL = 0,
		SCOPE_LOCAL = 1
	};
	class IMPLEventHandler
	{
    	    public:
		IMPLEventHandler() {}
		virtual ~IMPLEventHandler() {}
		virtual void bind(const Principal& /* princ */, 
			const char *  /* interface_name */, 
			const Object& /* Object		*/,
			Environment& ) {}

		virtual void unbind(const Principal& /* princ */,
			const char * /* interface_name */,
			const Object& /* object */ ) {}

		virtual void client_aborted(const Principal& princ, 
			const char *interface_name,
			const Object& object ) {
			unbind(princ, interface_name, object);
		}

		virtual void pre_method(const Principal&,
			const char * /* interface_name */,
			const Object& /* object */, 
			ULong /* method_id */,
			Environment& ) {} 

		virtual void post_method(const Principal&,
			const char * /* interface_name */,
			const Object& /* object */, 
			ULong /* method_id */) {}
				 
	};

	// CORBA Compliant methods.
	virtual Object			*create(const ReferenceData& id,
					const ImplementationDef& impl);

	virtual void		dispose(Object& obj);
	virtual ReferenceData  *id(const Object& obj);
	virtual void		change_implementation(const Object& obj,
					const ImplementationDef& impl);

	virtual const Principal	*principal(const Object& obj,
					const Environment& env);

	static void		impl_is_ready();
	virtual void		deactivate_impl(const ImplementationDef& impl);
	virtual void		obj_is_ready(const Object& obj,
					const ImplementationDef& impl);
	virtual void		deactivate_obj(const Object& obj);



	// Methods used by Skeletons generated by ORBELINE compiler
	static RegistrationScope scope() { return _scope; }
	static void scope(RegistrationScope scope) { _scope = scope; }

	static BOA		*instance();
	virtual void		register_implementation(Object *id,
				const char *interface_name,
				CORBA::ULong num_methods,
				_PMCSkelFunc  *func_list,
				void *obj,
				Environment& env,
				RegistrationScope reg_scope,
				IMPLEventHandler *handler = 
					(IMPLEventHandler *)NULL) =0; 

	void			register_implementation(Object *id,
				const char *interface_name,
				CORBA::ULong num_methods,
				_PMCSkelFunc  *func_list,
				void *obj,
				Environment& env,
				IMPLEventHandler *handler = 
					(IMPLEventHandler *)NULL) {
			register_implementation(id, interface_name,
				num_methods, func_list, obj, env, _scope, 
				handler);
	}
				

	virtual void		unregister_implementation(Object *id,
				const char *interface_name,
				void *obj,
				Environment& env) =0;

	virtual CORBA::Object	*implementation(const char *interface_name,
				const char *object_name) =0;

	// General methods that may be used by users
	virtual	CORBA::Boolean	is_persistent(const Object *obj);

	virtual void		event_handler(Object *id,
				Environment& env,
				IMPLEventHandler *handler) =0;

	virtual IMPLEventHandler *event_handler(Object *id,
				Environment& env) =0;

	void			global_handler(IMPLEventHandler *handler) {
				_handler = handler;
	}
	IMPLEventHandler 	*global_handler() { return _handler; }
				
    protected:
	IMPLEventHandler		*_handler;
	static RegistrationScope	_scope;	 // Default is global

	BOA() { _handler = (IMPLEventHandler *)NULL; }
	virtual ~BOA() {}

};

#endif
